@@ -91,7 +91,18 @@
 #define KERNEL_FEATURES 1
 #define INET 1
 
+#if NS_TARGET >= 40
+#if NS_TARGET >= 41
+#include <kernserv/lock.h>
+#else
+#include <kern/lock.h>
+#endif /* NS_TARGET */
+#endif /* NS_TARGET */
+
 #include <sys/param.h>
+#if NS_TARGET >= 41
+typedef simple_lock_data_t lock_data_t;		/* XXX */
+#endif /* NS_TARGET */
 #include <sys/proc.h>
 #include <sys/user.h>
 #include "netbuf.h"
@@ -103,8 +114,10 @@
 #include <sys/dk.h>
 #include <sys/uio.h>
 #include <sys/errno.h>
+#if !(NS_TARGET >= 40)
+/*  XXX what happened to this header file? */
 #include <machine/param.h>
-
+#endif
 
 #include <kernserv/prototypes.h>
 /* NeXT broke spl.h in 3.2/m68k. Thanks EPS! */
@@ -144,9 +157,17 @@ int	ppptioctl __P((struct tty *tp, int cmd, void *data, int flag));
 void	pppinput __P((int c, struct tty *tp));
 void	pppstart __P((struct tty *tp));
 
+/*
+ * Must return an actual netbuf_t since other protocols
+ * use this to get our buffers.
+ */
 netbuf_t	pppgetbuf __P((netif_t));
+
+/*
+ * Must accept an actual netbuf_t since others use this
+ * Procedure to access our output routine.
+ */
 int     	pppoutput __P((netif_t ifp, netbuf_t m, void *arg));
-void		pppintr __P((void *));
 
 static u_int16_t pppfcs __P((u_int16_t fcs, u_char *cp, int len));
 static void	pppasyncstart __P((struct ppp_softc *));
@@ -159,15 +180,6 @@ void	        ppplogchar __P((struct ppp_softc *, int));
 
 extern kern_server_t instance;
 
-#ifdef ADD_ERRORS
-
-int error_pkt_count = 0;   /* Number of packets received */
-int error_limit = -1;       /* Insert error at this limit */
-int random_count = 0;
-
-#endif /* ADD_ERRORS */
-
-
 /*
  * Does c need to be escaped?
  */
@@ -184,23 +196,23 @@ extern int ttymodem(struct tty*, int);
 extern int ttselect(struct tty *tp, int rw);
 
 
-static netbuf_t
+static NETBUF_T
 pppgetinbuf(netif_t ifp)
 {
     register struct ppp_softc *sc = &ppp_softc[if_unit(ifp)];
-    netbuf_t nb;
+    NETBUF_T nb;
     int len = MAX(sc->sc_mru, PPP_MTU) + sizeof (struct ifnet *) +
 #ifdef VJC
 	      VJ_HDRLEN +
 #endif
 	      PPP_HDRLEN + PPP_FCSLEN;
-    nb =  ppp_nb_alloc(len);
+    nb =  NB_ALLOC(len);
     if (nb != NULL)
       {
 #ifdef VJC
-	ppp_nb_shrink_top(nb, VJ_HDRLEN + PPP_HDRLEN);
+	NB_SHRINK_TOP(nb, VJ_HDRLEN + PPP_HDRLEN);
 #else
-	ppp_nb_shrink_top(nb, PPP_HDRLEN);
+	NB_SHRINK_TOP(nb, PPP_HDRLEN);
 #endif
       }
 
@@ -215,7 +227,7 @@ void
 pppfillfreeq(void *arg)
 {
     struct ppp_softc *sc = (struct ppp_softc *)arg;
-    netbuf_t nb;
+    NETBUF_T nb;
     volatile static int in = 0;
 
     if (in)
@@ -225,14 +237,6 @@ pppfillfreeq(void *arg)
     while(!nbq_high(&sc->sc_freeq)) {
 	nb = pppgetinbuf(sc->sc_if);
 	if (! nb) break;
-#if 0
-	/*
-	 * we no longer reset the length to 0 and then advance the
-	 * packet length bit by bit; instead we write into it whenever
-	 * we want, and at the end resize the packet before handing-off.
-	 */
-	nb_shrink_bot(nb, nb_size(nb));
-#endif
 	nbq_enqueue(&sc->sc_freeq, nb);
     }
 
@@ -275,7 +279,7 @@ pppopen(dev, tp)
     sc->sc_m = NULL;
     bzero(sc->sc_asyncmap, sizeof(sc->sc_asyncmap));
     sc->sc_asyncmap[0] = 0xffffffff;
-    sc->sc_asyncmap[3] = 0x60000000;
+    sc->sc_asyncmap[3] = 0x60000000;    /* 0x7D and 0x7E */
     sc->sc_rasyncmap = 0;
     sc->sc_devp = (void *) tp;
     sc->sc_start = pppasyncstart;
@@ -330,11 +334,11 @@ pppasyncrelinq(sc)
 
     s = splimp();
     if (sc->sc_outm) {
-	nb_free(sc->sc_outm);
+	NB_FREE(sc->sc_outm);
 	sc->sc_outm = NULL;
     }
     if (sc->sc_m) {
-	nb_free(sc->sc_m);
+	NB_FREE(sc->sc_m);
 	sc->sc_m = NULL;
     }
     if (sc->sc_flags & SC_TIMEOUT) {
@@ -353,12 +357,22 @@ pppread(tp, uio)
     struct uio *uio;
 {
     register struct ppp_softc *sc = (struct ppp_softc *)tp->t_sc;
-    netbuf_t m;
+    NETBUF_T m;
     register int s;
     int error = 0;
+    struct nty *np = ttynty(tp);
+
+#ifdef NEW_CLOCAL
+    if ((tp->t_state & TS_CARR_ON) == 0 && (np->t_pflags & TP_CLOCAL) == 0)
+	return 0;		/* end of file */
+
+#else
 
     if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_flags & CLOCAL) == 0)
 	return 0;		/* end of file */
+
+#endif /* NEW_CLOCAL */
+
     if (sc == NULL || tp != (struct tty *) sc->sc_devp)
 	return 0;
     s = splimp();
@@ -367,16 +381,7 @@ pppread(tp, uio)
 	    splx(s);
 	    return (EWOULDBLOCK);
 	}
-#if 0
-	/* NetBSD version... */
-	error = ttysleep(tp, (caddr_t)&tp->t_rawq, TTIPRI|PCATCH, ttyin, 0);
-	if (error) {
-	    splx(s);
-	    return error;
-	}
-#else
 	sleep((caddr_t)&tp->t_rawq, TTIPRI);
-#endif
     }
     if (tp->t_line != PPPDISC) {
 	splx(s);
@@ -394,8 +399,8 @@ pppread(tp, uio)
 	IOLogDbg("Read didn't get a buffer at %s %d\n", __FILE__, __LINE__);
       return -1;
     }
-    error = uiomove(nb_map(m), nb_size(m), UIO_READ, uio);
-    nb_free(m);
+    error = uiomove(NB_MAP(m), NB_SIZE(m), UIO_READ, uio);
+    NB_FREE(m);
     return (error);
 }
 
@@ -408,12 +413,23 @@ pppwrite(tp, uio)
     struct uio *uio;
 {
     register struct ppp_softc *sc = (struct ppp_softc *)tp->t_sc;
-    netbuf_t m;
+    NETBUF_T m;
     struct sockaddr dst;
     int len, error;
+    struct nty *np = ttynty(tp);
+
+#ifdef NEW_CLOCAL
+
+    if ((tp->t_state & TS_CARR_ON) == 0 && (np->t_pflags & TP_CLOCAL) == 0)
+	return 0;		/* wrote 0 bytes */
+
+#else
 
     if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_flags & CLOCAL) == 0)
 	return 0;		/* wrote 0 bytes */
+
+#endif /* NEW_CLOCAL */
+
     if (tp->t_line != PPPDISC)
 	return (EINVAL);
     if (sc == NULL || tp != (struct tty *) sc->sc_devp)
@@ -421,23 +437,25 @@ pppwrite(tp, uio)
     if (uio->uio_resid > if_mtu(sc->sc_if) + PPP_HDRLEN ||
       uio->uio_resid < PPP_HDRLEN)
 	return (EMSGSIZE);
-    m = pppgetbuf(sc->sc_if);
+    m = nb_TO_NB(pppgetbuf(sc->sc_if));
+
     if (m == NULL){
       if (sc->sc_flags & SC_DEBUG)
 	IOLogDbg("No buffers available for user level write()\n");
       return(ENOBUFS);
     }
-    ppp_nb_grow_top(m, PPP_HDRLEN);
+    NB_GROW_TOP(m, PPP_HDRLEN);
     len = uio->uio_resid;
-    if (error = uiomove(nb_map(m), nb_size(m), UIO_WRITE, uio)) {
-	nb_free(m);
+    if (error = uiomove(NB_MAP(m), NB_SIZE(m), UIO_WRITE, uio)) {
+	NB_FREE(m);
 	return error;
     }
-    nb_shrink_bot(m, nb_size(m) - len);
+    NB_SHRINK_BOT(m, NB_SIZE(m) - len);
     dst.sa_family = AF_UNSPEC;
     bcopy(mtod(m, u_char *), dst.sa_data, PPP_HDRLEN);
-    ppp_nb_shrink_top(m, PPP_HDRLEN);
-    return (pppoutput(sc->sc_if, m, &dst));
+
+    NB_SHRINK_TOP(m, PPP_HDRLEN);
+    return (pppoutput(sc->sc_if, NB_TO_nb(m), &dst));
 }
 
 /*
@@ -463,34 +481,22 @@ ppptioctl(tp, cmd, data, flag)
     case PPPIOCSASYNCMAP:
 	if (! suser())
 	    return EPERM;
-#if 0
-	IOLogDbg("ppp%d: set async map to 0x%08x\n", if_unit(sc->sc_if),
-		 *(u_long *)data);
-#endif
+
 	sc->sc_asyncmap[0] = *(u_int *)data;
 	break;
 
     case PPPIOCGASYNCMAP:
 	*(u_int *)data = sc->sc_asyncmap[0];
-#if 0
-	IOLogDbg("ppp%d: asyncmap gets 0x%x\n", if_unit(sc->sc_if), *(u_int *)data);
-#endif
 	break;
 
     case PPPIOCSRASYNCMAP:
 	if (! suser())
 	    return EPERM;
 	sc->sc_rasyncmap = *(u_int *)data;
-#if 0
-	IOLogDbg("ppp%d: setting rasyncmap 0x%x\n", if_unit(sc->sc_if), sc->sc_rasyncmap);
-#endif
 	break;
 
     case PPPIOCGRASYNCMAP:
 	*(u_int *)data = sc->sc_rasyncmap;
-#if 0
-	IOLogDbg("ppp%d: rasyncmap gets 0x%x\n", if_unit(sc->sc_if), *(u_int *)data);
-#endif
 	break;
 
     case PPPIOCSXASYNCMAP:
@@ -506,9 +512,6 @@ ppptioctl(tp, cmd, data, flag)
 
     case PPPIOCGXASYNCMAP:
 	bcopy(sc->sc_asyncmap, data, sizeof(sc->sc_asyncmap));
-#if 0
-	IOLogDbg("ppp%d: xasyncmap gets 0x%x/0x%x/0x%x/0x%x\n", if_unit(sc->sc_if), ((u_int *)data)[0], ((u_int *)data)[1], ((u_int *)data)[2], ((u_int *)data)[3]);
-#endif
 	break;
 
     default:
@@ -622,12 +625,22 @@ pppstart(tp)
     register struct tty *tp;
 {
     register struct ppp_softc *sc = (struct ppp_softc *) tp->t_sc;
-    register netbuf_t m;
+    register NETBUF_T m;
     register int len;
     register u_char *start, *stop, *cp;
     int n, ndone, done, idle;
+    struct nty *np = ttynty(tp);
+
+#ifdef NEW_CLOCAL
+
+    if ((tp->t_state & TS_CARR_ON) == 0 && (np->t_pflags & TP_CLOCAL) == 0
+
+#else
+
+    if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_flags & CLOCAL) == 0 
+
+#endif /* NEW_CLOCAL */
 
-    if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_flags & CLOCAL) == 0
 	|| sc == NULL || tp != (struct tty *) sc->sc_devp) {
 	if (tp->t_oproc != NULL)
 	    (*tp->t_oproc)(tp);
@@ -666,16 +679,17 @@ pppstart(tp)
 	    }
 
 	    /* Calculate the FCS for the first netbuf's worth. */
-	    sc->sc_outfcs = pppfcs(PPP_INITFCS, mtod(m, u_char *), nb_size(m));
+	    sc->sc_outfcs = pppfcs(PPP_INITFCS, mtod(m, u_char *), NB_SIZE(m));
 	    sc->sc_outfcs ^= 0xffff;
-	    cp = mtod(m, u_char *) + nb_size(m);
-	    nb_grow_bot(m, PPP_FCSLEN);
+	    
+	    cp = mtod(m, u_char *) + NB_SIZE(m);
+	    NB_GROW_BOT(m, PPP_FCSLEN);
 	    *cp++ = sc->sc_outfcs & 0xFF;
 	    *cp++ = (sc->sc_outfcs >> 8) & 0xFF;
 	}
 
 	start = mtod(m, u_char *);
-	len = nb_size(m);
+	len = NB_SIZE(m);
 	stop = start + len;
 	while (len > 0) {
 	    /*
@@ -685,9 +699,16 @@ pppstart(tp)
 	    for (cp = start; cp < stop; cp++)
 		if (ESCAPE_P(*cp))
 		    break;
+
 	    n = cp - start;
+
 	    if (n) {
-		/* NetBSD (0.9 or later), 4.3-Reno or similar. */
+		/*
+		 * b_to_q returns the number of characters
+		 * _not_ sent
+		 *
+		 * NetBSD (0.9 or later), 4.3-Reno or similar.
+		 */
 		ndone = n - b_to_q(start, n, &tp->t_outq);
 		len -= ndone;
 		start += ndone;
@@ -695,6 +716,7 @@ pppstart(tp)
 
 		if (ndone < n)
 		    break;	/* packet doesn't fit */
+
 	    }
 
 	    /*
@@ -721,7 +743,7 @@ pppstart(tp)
 
 	if (!done) {
 	    /* remember where we got to */
-	    ppp_nb_shrink_top(m, start - mtod(m, u_char *));
+	    NB_SHRINK_TOP(m, start - mtod(m, u_char *));
 	    break;	/* can't do any more at the moment */
 	}
 
@@ -730,13 +752,14 @@ pppstart(tp)
 	 * We make the length zero in case the flag
 	 * cannot be output immediately.
 	 */
-	ppp_nb_shrink_top(m, nb_size(m));
+	NB_SHRINK_TOP(m, NB_SIZE(m));
 	if (putc(PPP_FLAG, &tp->t_outq))
 	    break;
 	sc->sc_bytessent++;
 
+
 	/* Finished with this netbuf; free it and move on. */
-	nb_free(m);
+	NB_FREE(m);
 	m = NULL;
 	incr_cnt(sc->sc_if, if_opackets);
 
@@ -756,7 +779,11 @@ pppstart(tp)
      * drained the t_outq.
      */
     if (!idle && (sc->sc_flags & SC_TIMEOUT) == 0) {
+#if NS_TARGET >= 40
+	timeout(ppp_timeout, (void *) sc, 1);
+#else
 	ns_timeout(ppp_timeout, (void *) sc, 1 * (1000000000L / HZ), CALLOUT_PRI_SOFTINT0);
+#endif /*NS_TARGET */
 	sc->sc_flags |= SC_TIMEOUT;
     }
 
@@ -799,31 +826,107 @@ pppgetm(sc)
      * hand-shake is lock-step (ie. single packet).
      */
     if (sc->sc_m != NULL)
-	nb_free(sc->sc_m);
+	NB_FREE(sc->sc_m);
     sc->sc_m = nbq_dequeue(&sc->sc_freeq);
     splx(s);
 }
 
 /*
- * Currently only called by the NeXT supplied serial drivers.
- * This is a prime candidate for optimization.
+ * 4.3 says this is an unused function.  However,
+ * it appears to be returning a NULL terminated string
+ * of several characters.  My guess is that the serial
+ * driver is doing a little buffering so that we don't
+ * get burdend with interrupts.
+ *
+ * This function gets called when you use the NeXT
+ * supplied serial drivers.  It does not get called
+ * with the MuX driver.  
+ *
+ * In order to expedite the work done here, we
+ * handle most things here that don't require
+ * processing of a PPP_FLAG.
+ *
  */
 
 void
 ppprend(cp, n, tp)
-    char *cp;
+    unsigned char *cp;
     int n;
     struct tty *tp;
 {
 
-  static count = 0;
-
-/*
-  if (count == 0)
-    IOLog("Optomize ppprend!\n");
-*/
-	
+#ifndef OPTIMIZE_PPPREND	
+#warning PPPREND Not optimized!!!
   while (n--) pppinput((u_char) *cp++, tp);
+#else
+
+
+  register struct ppp_softc *sc = (struct ppp_softc *)tp->t_sc;
+  register int ret;
+
+  if (sc == NULL || tp != (struct tty *) sc->sc_devp)
+    {
+      printf("Warning, bad softc structure at %s %d\n", __FILE__, __LINE__);
+      return;
+    }
+
+  /*
+   * We can handle FLUSHs, ESCAPES, and non PPP_FLAG characters
+   */
+  
+  while (n)
+    {
+      if (sc->sc_flags & SC_FLUSH)
+	{
+	  do
+	    {
+	      if (*(cp++) == PPP_FLAG)
+		{
+		  pppinput(PPP_FLAG, tp);
+		  --n;
+		  break;
+		}
+	      else if (sc->sc_flags & SC_LOG_FLUSH)
+		ppplogchar(sc, *cp);
+	    }
+	  while(--n);
+	}
+      else if (sc->sc_ilen > 3 &&
+	       (NB_SIZE(sc->sc_m) - sc->sc_ilen) > n &&
+	       *cp != PPP_FLAG &&
+	       *cp != PPP_ESCAPE)        /* Dont really handle escapes properly...should */
+	{
+	  unsigned char* cp1 = cp;
+	  if (sc->sc_flags & SC_ESCAPED)
+	    {
+	      sc->sc_flags &= ~SC_ESCAPED;
+	      *cp ^= PPP_TRANS;
+	    }
+	  
+	  do
+	    {
+	      sc->sc_fcs = PPP_FCS(sc->sc_fcs, *(cp++));
+	      if (sc->sc_flags & SC_LOG_RAWIN)
+		ppplogchar(sc, *cp);
+	      
+	    } while(--n && *cp != PPP_FLAG && *cp != PPP_ESCAPE);
+	  
+	  
+	  bcopy(cp1, sc->sc_mp, (cp-cp1));
+	  
+	  sc->sc_bytesrcvd += (cp - cp1);
+	  sc->sc_ilen += (cp-cp1);
+	  sc->sc_mp += (cp-cp1);
+	  
+	}
+      else
+	{
+	  --n;
+	  pppinput(*(cp++), tp);
+	}
+    }
+  
+#endif /* OPTIMIZE_PPPREND */
 }
 
 /*
@@ -840,14 +943,13 @@ pppinput(c, tp)
     register struct tty *tp;
 {
     register struct ppp_softc *sc;
-    netbuf_t m;
+    NETBUF_T m;
     int ilen, s;
 
     sc = (struct ppp_softc *) tp->t_sc;
     if (sc == NULL || tp != (struct tty *) sc->sc_devp)
 	return;
 
-/*    s = spltty();    +++SJP This cannot be reentrant */
     ++tk_nin;
     ++sc->sc_bytesrcvd;
 
@@ -872,106 +974,58 @@ pppinput(c, tp)
 	ppplogchar(sc, c);
 
     if (c == PPP_FLAG) {
+
+      if (sc->sc_ilen == 0)
+	return;
+
 	ilen = sc->sc_ilen;
 	sc->sc_ilen = 0;
 
 	if (sc->sc_rawin_count > 0)
 	    ppplogchar(sc, -1);
 
-#ifdef ADD_ERRORS
-      /*
-       * This section will add (very poor pseudo) random packet
-       * errors if defined.
-       */
-
-      if (error_limit == -1)
-	{
-	  error_limit = 30;
-	  IOLog("Introducing random error after %d more packets\n",
-		 error_limit);
-	}	
-
-      if (error_pkt_count >=  error_limit)
-	{
-	  sc->sc_fcs = !PPP_GOODFCS;
-
-	  error_pkt_count = 0;
-	 switch (error_limit)
-	    {
-	    case 1:
-	      ++random_count;
-	      if (random_count > 5)
-		{
-		  error_limit = 1;
-		  random_count = 0;
-		}
-	      else
-		error_limit = 2;
-	      break;
-	    case 2:
-	      error_limit = 3;
-	      break;
-	    case 3:
-	      ++random_count;
-	      if (random_count > 5)
-		{
-		  error_limit = 1;
-		  random_count = 0;
-		}
-	      else
-		error_limit = 10;
-	      break;
-	    case 10:
-	      ++random_count;
-	      error_limit = 30;
-	      break;
-	    case 30:
-	      error_limit = 1;
-	      break;
-	    default:
-	      error_limit = 1;
-	    }
-
-	  IOLog("Introducing random error after %d more packets\n",
-		 error_limit);
-	}
-      else
-	++error_pkt_count;
-      
-
-#endif /* ADD_ERRORS */      
-
 	/*
-	 * If SC_ESCAPED is set, then we've seen the packet
+	 * From the RFC:
+	 *  Each Control Escape octet is also
+         *  removed, and the following octet is exclusive-or'd with hexadecimal
+         *  0x20, unless it is the Flag Sequence (which aborts a frame).
+         *
+	 * So, if SC_ESCAPED is set, then we've seen the packet
 	 * abort sequence "}~".
 	 */
-	if (sc->sc_flags & (SC_FLUSH | SC_ESCAPED)
-	    || ilen > 0 && sc->sc_fcs != PPP_GOODFCS) {
+	if ((sc->sc_flags & (SC_FLUSH | SC_ESCAPED)) ||
+	    ((ilen > 0) && (sc->sc_fcs != PPP_GOODFCS)))
+	  {
 	    sc->sc_flags |= SC_PKTLOST;	/* note the dropped packet */
-	    if ((sc->sc_flags & (SC_FLUSH | SC_ESCAPED)) == 0){
-		IOLogDbg("ppp%d: bad fcs 0x%04x\n", if_unit(sc->sc_if), sc->sc_fcs);
+	    if ((sc->sc_flags & (SC_FLUSH | SC_ESCAPED)) == 0)
+	      {
+		IOLog("ppp%d: bad fcs 0x%04x\n", if_unit(sc->sc_if), sc->sc_fcs);
 		incr_cnt(sc->sc_if, if_ierrors);
-	    } else
+	      }
+	    else
+	      {
+		IOLog("ppp%d: bad packet flushed...\n", if_unit(sc->sc_if));
 		sc->sc_flags &= ~(SC_FLUSH | SC_ESCAPED);
-/*	    splx(s); */
+	      }
 	    return;
-	}
-
-	if (ilen < PPP_HDRLEN + PPP_FCSLEN) {
-	    if (ilen) {
+	  }
+	
+	if (ilen < (PPP_HDRLEN + PPP_FCSLEN))
+	  {
+	    if (ilen)
+	      {
 		IOLogDbg("ppp%d: too short (%d)\n", if_unit(sc->sc_if), ilen);
 		incr_cnt(sc->sc_if, if_ierrors);
 		sc->sc_flags |= SC_PKTLOST;
-	    }
-/*	    splx(s); */
+	      }
 	    return;
-	}
-
+	  }
+	
 	/*
 	 * Remove FCS trailer.  Set packet length...
 	 */
 	ilen -= PPP_FCSLEN;
-	nb_shrink_bot(sc->sc_m, nb_size(sc->sc_m) - ilen);
+	NB_SHRINK_BOT(sc->sc_m, NB_SIZE(sc->sc_m) - ilen);
 
 	/* excise this netbuf */
 	m = sc->sc_m;
@@ -981,19 +1035,25 @@ pppinput(c, tp)
 	sc->sc_flags &= ~SC_PKTLOST;
 
 	pppgetm(sc);
-/*	splx(s); */
 	return;
     }
 
     if (sc->sc_flags & SC_FLUSH) {
 	if (sc->sc_flags & SC_LOG_FLUSH)
 	    ppplogchar(sc, c);
-/*	splx(s); */
 	return;
     }
 
+/*
+ * From the RFC:
+ *  On reception, prior to FCS computation, each octet with value less
+ *  than hexadecimal 0x20 is checked.  If it is flagged in the receiving
+ *  ACCM, it is simply removed (it may have been inserted by intervening
+ *  data communications equipment).  Each Control Escape octet is also
+ *  removed, and the following octet is exclusive-or'd with hexadecimal
+ *  0x20, unless it is the Flag Sequence (which aborts a frame).
+ */
     if (c < 0x20 && (sc->sc_rasyncmap & (1 << c))) {
-/*	splx(s); */
 	return;
     }
 
@@ -1002,7 +1062,6 @@ pppinput(c, tp)
 	c ^= PPP_TRANS;
     } else if (c == PPP_ESCAPE) {
 	sc->sc_flags |= SC_ESCAPED;
-/*	splx(s); */
 	return;
     }
 
@@ -1016,6 +1075,7 @@ pppinput(c, tp)
      * Fourth octet is second octet of protocol.
      */
     if (sc->sc_ilen == 0) {
+
 	/* reset the input netbuf */
 	if (sc->sc_m == NULL) {
 	    pppgetm(sc);
@@ -1032,6 +1092,7 @@ pppinput(c, tp)
 	m = sc->sc_m;
 	sc->sc_mp = mtod(m, char *);
 	sc->sc_fcs = PPP_INITFCS;
+
 	if (c != PPP_ALLSTATIONS) {
 	    if (sc->sc_flags & SC_REJ_COMP_AC) {
 		IOLogDbg("ppp%d: garbage received: 0x%02x (need 0x%02x)\n",
@@ -1043,16 +1104,20 @@ pppinput(c, tp)
 	    sc->sc_ilen += 2;
 	}
     }
+
+
     if (sc->sc_ilen == 1 && c != PPP_UI) {
 	IOLogDbg("ppp%d: missing UI (0x%02x), got 0x%02x\n",
 		   if_unit(sc->sc_if), PPP_UI, c);
 	goto flush;
     }
+
     if (sc->sc_ilen == 2 && (c & 1) == 1) {
 	/* a compressed protocol */
 	*sc->sc_mp++ = 0;
 	sc->sc_ilen++;
     }
+
     if (sc->sc_ilen == 3 && (c & 1) == 0) {
 	IOLogDbg("ppp%d: bad protocol %x\n", if_unit(sc->sc_if),
 		   (sc->sc_mp[-1] << 8) + c);
@@ -1069,7 +1134,6 @@ pppinput(c, tp)
     /* ilen was incremented above... */
     *sc->sc_mp++ = c;
     sc->sc_fcs = PPP_FCS(sc->sc_fcs, c);
-/*    splx(s); */
     return;
 
  flush:
@@ -1079,7 +1143,6 @@ pppinput(c, tp)
 	if (sc->sc_flags & SC_LOG_FLUSH)
 	    ppplogchar(sc, c);
     }
-/*    splx(s); */
     return;
 }
 
